@@ -48,7 +48,7 @@ IGNORABLE_FIELDS = frozenset((
 class BugzillaError(WebScraperError):
     '''Generic error'''
     def __init__(self, error, bug_id=None):
-        super(BugzillaError, self).__init__()
+        super(BugzillaError, self).__init__(error)
         self.bug_id = bug_id
         self.error = error
 
@@ -224,6 +224,49 @@ class Bugzilla(WebScraper):
         )
         self.logger = logging.getLogger('suse.bugzilla')
 
+    def possible_relogin(self, error):
+        """
+        Logins again to workaround possible bad cookies.
+        """
+        if (error.original
+                and hasattr(error.original, 'code')
+                and error.original.code == 502
+                and self.cookie_set):
+            self.logger.warning(
+                'Got 502 (Bad Gateway), clearing cookies and loging in again'
+            )
+            self.cookie_set = False
+            self.cookiejar.clear()
+            self.login(force=True)
+            return True
+        return False
+
+    def request(self, action, paramlist=None, **kwargs):
+        '''
+        Performs single request on a server (loads single page).
+        '''
+        try:
+            return super(Bugzilla, self).request(
+                action, paramlist, **kwargs
+            )
+        except WebScraperError as error:
+            if self.possible_relogin(error):
+                return super(Bugzilla, self).request(
+                    action, paramlist, **kwargs
+                )
+            raise error
+
+    def submit(self):
+        '''
+        Submits currently selected browser form.
+        '''
+        try:
+            return super(Bugzilla, self).submit()
+        except WebScraperError as error:
+            if self.possible_relogin(error):
+                return super(Bugzilla, self).submit()
+            raise error
+
     def check_viewing_html(self):
         '''
         Checks whether the browser is in HTML viewing state.
@@ -247,7 +290,7 @@ class Bugzilla(WebScraper):
         except LinkNotFoundError:
             return False
 
-    def login(self):
+    def login(self, force=False):
         '''
         Login to Bugzilla using Access Manager.
         '''
@@ -257,7 +300,7 @@ class Bugzilla(WebScraper):
         try:
             # Submit fake javascript form
             self.browser.select_form(nr=0)
-            response = self._submit()
+            response = self.submit()
 
             # Find the login form
             self.browser.select_form(nr=0)
@@ -268,7 +311,7 @@ class Bugzilla(WebScraper):
             raise BugzillaLoginFailed('Failed to parse HTML for login!')
 
         self.logger.info('Doing login')
-        response = self._submit()
+        response = self.submit()
 
         text = response.read()
 
@@ -425,6 +468,36 @@ class Bugzilla(WebScraper):
             ('chfieldfrom', startdate.strftime('%Y-%m-%d %H:%M:%S +0000'))
         ])
 
+    def get_opensec_bugs(self):
+        '''
+        Searches for security related bugs
+        '''
+        return self.do_search([
+            ('short_desc', '^VUL-[0-9]'),
+            ('query_format', 'advanced'),
+            ('bug_status', 'NEW'),
+            ('bug_status', 'ASSIGNED'),
+            ('bug_status', 'NEEDINFO'),
+            ('bug_status', 'REOPENED'),
+            ('component', 'Incidents'),
+            ('product', 'SUSE Security Incidents'),
+            ('short_desc_type', 'regexp')
+        ])
+
+    def get_recent_secbugs(self, startdate):
+        '''
+        Returns lis of security bugs changed since start date.
+        '''
+        return self.do_search([
+            ('short_desc', '^VUL-[0-9]'),
+            ('query_format', 'advanced'),
+            ('short_desc_type', 'regexp'),
+            ('chfieldto', 'Now'),
+            ('component', 'Incidents'),
+            ('product', 'SUSE Security Incidents'),
+            ('chfieldfrom', startdate.strftime('%Y-%m-%d %H:%M:%S +0000'))
+        ])
+
     def get_openl3_bugs(self):
         '''
         Searches for bugs with openL3 in whiteboard.
@@ -477,11 +550,10 @@ class Bugzilla(WebScraper):
         # Convert to integers
         return [int(x) for x in match]
 
-    def update_bug(self, bugid, callback=None, callback_param=None,
-                   whiteboard_add=None, whiteboard_remove=None, **kwargs):
-        '''
-        Updates bugzilla.
-        '''
+    def load_update_form(self, bugid):
+        """
+        Selects form for bug update.
+        """
         if self.anonymous:
             raise BugzillaUpdateError('No updates in anonymous mode!')
 
@@ -495,14 +567,26 @@ class Bugzilla(WebScraper):
 
         self.check_viewing_html()
 
-        changes = False
-
         # Find the form
         try:
             self.browser.select_form(name="changeform")
         except FormNotFoundError:
             raise BugzillaUpdateError('Failed to parse HTML to update bug!')
 
+    def update_bug(self, bugid, callback=None, callback_param=None,
+                   whiteboard_add=None, whiteboard_remove=None, **kwargs):
+        '''
+        Updates bugzilla.
+        '''
+        self.load_update_form(bugid)
+
+        changes = False
+
+        # Do not add ourselves to CC when setting whiteboard
+        if ((whiteboard_add is not None or whiteboard_remove is not None)
+                and 'addselfcc' not in kwargs):
+            kwargs['addselfcc'] = []
+
         # Set parameters
         for k in kwargs:
             val = kwargs[k]
@@ -531,7 +615,7 @@ class Bugzilla(WebScraper):
             return
 
         # Submit
-        resp = self._submit()
+        resp = self.submit()
         data = resp.read()
         if data.find('Mid-air collision!') != -1:
             raise BugzillaUpdateError('Mid-air collision!')
@@ -554,12 +638,6 @@ class Bugzilla(WebScraper):
 
         self.browser['status_whiteboard'] = whiteboard
 
-        # Do not add ourselves to cc
-        try:
-            self.browser['addselfcc'] = []
-        except ControlNotFoundError:
-            pass
-
         return changes
 
 
@@ -578,7 +656,7 @@ class APIBugzilla(Bugzilla):
         else:
             self.browser.add_password(base + '/', user, password)
 
-    def login(self):
+    def login(self, force=False):
         '''
         Checks login to Bugzilla using HTTP authentication.
         '''
@@ -612,13 +690,28 @@ class DjangoBugzilla(APIBugzilla):
         mail_admins(subject, message, fail_silently=True)
         super(DjangoBugzilla, self).log_parse_error(bugid, data)
 
+    def login(self, force=False):
+        """
+        Login with caching cookies in Django.
+        """
+        from django.core.cache import cache
+        if force:
+            cookies = None
+        else:
+            cookies = cache.get('bugzilla-access-cookies')
+
+        if cookies is None:
+            super(DjangoBugzilla, self).login(force)
+            cache.set('bugzilla-access-cookies', self.get_cookies())
+        else:
+            self.set_cookies(cookies)
+
 
 def get_django_bugzilla():
     '''
     Returns logged in bugzilla object. Access cookies are stored in django
     cache.
     '''
-    from django.core.cache import cache
     from django.conf import settings
     bugzilla = DjangoBugzilla(
         settings.BUGZILLA_USERNAME,
@@ -630,12 +723,6 @@ def get_django_bugzilla():
     if settings.BUGZILLA_USERNAME == '':
         return bugzilla
 
-    cookies = cache.get('bugzilla-access-cookies')
-
-    if cookies is None:
-        bugzilla.login()
-        cache.set('bugzilla-access-cookies', bugzilla.get_cookies())
-    else:
-        bugzilla.set_cookies(cookies)
+    bugzilla.login()
 
     return bugzilla
